package com.guams.review.configuration;

import com.guams.review.service.AuthorService;
import io.jsonwebtoken.JwtException;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.time.Instant;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

@Component
public class JwtAuthenticationFilter extends OncePerRequestFilter {

    private final JwtTokenUtil jwtTokenUtil;
    private final AuthorService authorService;

    // Map pour traquer les requêtes : UUID -> { dernière réinitialisation, compteur de requêtes }
    private final ConcurrentHashMap<String, RateLimiter> rateLimiters = new ConcurrentHashMap<>();

    // Configuration du rate limiting
    private static final int MAX_REQUESTS = 5;
    private static final long TIME_WINDOW_MS = 60 * 1000; // 1 minute

    public JwtAuthenticationFilter(JwtTokenUtil jwtTokenUtil, AuthorService authorService) {
        this.jwtTokenUtil = jwtTokenUtil;
        this.authorService = authorService;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
        final String authHeader = request.getHeader("Authorization");

        if (authHeader != null && authHeader.startsWith("Bearer ")) {
            String token = authHeader.substring(7); // Retire le préfixe "Bearer "
            try {
                String authorUuid = jwtTokenUtil.extractAuthorId(token); // Récupère l'UUID du token

                if (authorUuid != null && SecurityContextHolder.getContext().getAuthentication() == null) {
                    UserDetails userDetails = authorService.loadUserByUsername(authorUuid); // Charge l'auteur

                    if (jwtTokenUtil.validateToken(token, userDetails, authorUuid)) {
                        // Applique le rate limiting
                        if (isRateLimited(authorUuid, request)) {
                            response.setStatus(429); // TOO MANY REQUESTS
                            response.getWriter().write("Rate limit exceeded. Please try again later.");
                            return;
                        }

                        UsernamePasswordAuthenticationToken authentication =
                                new UsernamePasswordAuthenticationToken(userDetails, null, userDetails.getAuthorities());
                        authentication.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
                        SecurityContextHolder.getContext().setAuthentication(authentication); // Met à jour le contexte
                    }
                }
            } catch (JwtException e) {
                response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
                return;
            }
        }

        filterChain.doFilter(request, response); // Continue le traitement
    }

    private boolean isRateLimited(String authorUuid, HttpServletRequest request) {
        // Ne limite que les requêtes POST
        if (!"POST".equalsIgnoreCase(request.getMethod())) {
            return false;
        }

        // Récupère ou crée un RateLimiter pour l'utilisateur
        RateLimiter limiter = rateLimiters.computeIfAbsent(authorUuid, k -> new RateLimiter());

        synchronized (limiter) {
            long now = Instant.now().toEpochMilli();
            if (now - limiter.lastReset > TIME_WINDOW_MS) {
                // Réinitialise la fenêtre de temps et le compteur
                limiter.lastReset = now;
                limiter.requestCount.set(0);
            }

            if (limiter.requestCount.incrementAndGet() > MAX_REQUESTS) {
                return true; // Limite atteinte
            }
        }

        return false;
    }

    // Classe interne pour suivre les requêtes
    private static class RateLimiter {
        private long lastReset;
        private AtomicInteger requestCount;

        public RateLimiter() {
            this.lastReset = Instant.now().toEpochMilli();
            this.requestCount = new AtomicInteger(0);
        }
    }
}
